from datetime import datetime
from typing import ClassVar, Optional

from pydantic import constr

from ..node import OntolocyNode
from ..relationship import OntolocyRelationship
from .cpe import CPE
from .cwe import CWE


class CVE(OntolocyNode):
    __primaryproperty__: ClassVar[str] = "cve_id"
    __primarylabel__: ClassVar[Optional[str]] = "CVE"

    cve_id: constr(to_upper=True, regex=r"CVE-\d{4}-\d{4,8}")  # noqa: F722
    published_date: Optional[datetime]
    assigner: Optional[str]
    description: Optional[str]


#
# OUTGOING RELATIONSHIPS
#


class CVERelatesToCWE(OntolocyRelationship):
    source: CVE
    target: CWE

    __relationshiptype__: ClassVar[str] = "CVE_RELATES_TO_CWE"


class CVERelatesToCPE(OntolocyRelationship):
    source: CVE
    target: CPE

    __relationshiptype__: ClassVar[str] = "CVE_RELATES_TO_CPE"



import hashlib
from typing import ClassVar, Optional

from pydantic import validator

from ..node import OntolocyNode


class Banner(OntolocyNode):

    __primaryproperty__: ClassVar[str] = "sha1"
    __primarylabel__: ClassVar[str] = "Banner"

    banner: str
    sha1: Optional[str] = None

    @validator("sha1", always=True)
    def set_sha1(cls, v, values):
        if v is None and "banner" in values:
            return hashlib.sha1(values["banner"].encode()).hexdigest()
        else:
            return v



from datetime import datetime
from typing import Any, ClassVar, Dict, Optional
from uuid import UUID

from pydantic import validator

from ..node import OntolocyNode
from ..relationship import OntolocyRelationship
from ..utils import generate_deterministic_uuid


class DNSRecord(OntolocyNode):
    __primaryproperty__: ClassVar[str] = "unique_id"
    __primarylabel__: ClassVar[Optional[str]] = "DNSRecord"

    type: str
    name: str
    content: str

    unique_id: Optional[UUID]

    @validator("unique_id", always=True)
    def generate_dnsrecord_uuid(cls, v: Optional[UUID], values: Dict[str, Any]) -> UUID:
        if v is None:
            key_values = [
                values["type"],
                values["name"],
                values["content"],
            ]

            v = generate_deterministic_uuid(key_values)

        return v


class DNSRecordPointsToIPAddress(OntolocyRelationship):
    __relationshiptype__: ClassVar[str] = "DNS_RECORD_POINTS_TO_IP_ADDRESS"

    source: DNSRecord
    target: "IPAddressNode"

    observation_date: datetime


class DNSRecordPointsToDomainName(OntolocyRelationship):
    __relationshiptype__: ClassVar[str] = "DNS_RECORD_POINTS_TO_DOMAIN_NAME"

    source: DNSRecord
    target: "DomainName"

    observation_date: datetime


class DNSRecordForDomain(OntolocyRelationship):
    __relationshiptype__: ClassVar[str] = "DNS_RECORD_FOR_DOMAIN"

    source: DNSRecord
    target: "DomainName"

    observation_date: datetime


from .domainname import DomainName  # noqa: E402

DNSRecordPointsToDomainName.update_forward_refs()
DNSRecordForDomain.update_forward_refs()

from .ip import IPAddressNode  # noqa: E402

DNSRecordPointsToIPAddress.update_forward_refs()



from typing import ClassVar, Optional

from pydantic import constr, validator

from ..node import OntolocyNode
from ..relationship import OntolocyRelationship
from .host import Host


class MACAddress(OntolocyNode):
    __primaryproperty__: ClassVar[str] = "mac_address"
    __primarylabel__: ClassVar[Optional[str]] = "MACAddress"

    mac_address: constr(
        regex=r"^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$"  # noqa: F722
    )

    @validator("mac_address", always=True)
    def format_mac_address(cls, v: str) -> str:
        """We want to standardize on colons and uppercase letters"""

        v = v.upper()
        v = v.replace("-", ":")

        return v


class MACAddressAssignedToHost(OntolocyRelationship):
    __relationshiptype__: ClassVar[str] = "MAC_ADDRESS_ASSIGNED_TO_HOST"

    source: MACAddress
    target: Host

    interface: Optional[str]



from datetime import datetime
from typing import Any, ClassVar, Dict, Optional
from uuid import UUID

from pydantic import HttpUrl, validator

from ..node import OntolocyNode
from ..relationship import OntolocyRelationship
from ..utils import generate_deterministic_uuid
from .cve import CVE


class Exploit(OntolocyNode):

    __primaryproperty__: ClassVar[str] = "unique_id"
    __primarylabel__: ClassVar[Optional[str]] = "Exploit"

    name: str
    published_date: Optional[datetime]
    url: HttpUrl

    unique_id: Optional[UUID] = None

    @validator("unique_id", always=True)
    def generate_unique_id(cls, v: Optional[UUID], values: Dict[str, Any]) -> UUID:

        if v is None:

            key_values = [
                values["name"],
                values["url"],
            ]

            v = generate_deterministic_uuid(key_values)

        return v


#
# OUTGOING RELATIONSHIPS
#


class ExploitExploitsVulnerability(OntolocyRelationship):
    source: Exploit
    target: CVE

    __relationshiptype__: ClassVar[str] = "EXPLOIT_EXPLOITS_VULNERABILITY"



from enum import Enum
from typing import Any, ClassVar, Dict, Optional
from uuid import UUID

from pydantic import IPvAnyAddress, validator

from ontolocy.models.port import Port
from ontolocy.node import OntolocyNode
from ontolocy.relationship import OntolocyRelationship
from ontolocy.utils import generate_deterministic_uuid

from .banner import Banner
from .cpe import CPE
from .jarmhash import JarmHash
from .url import URLNode
from .x509certificate import X509Certificate


class ListeningSocketProtocolEnum(str, Enum):
    tcp = "tcp"
    udp = "udp"
    sctp = "sctp"


class ListeningSocket(OntolocyNode):

    __primaryproperty__: ClassVar[str] = "unique_id"
    __primarylabel__: ClassVar[Optional[str]] = "ListeningSocket"

    protocol: ListeningSocketProtocolEnum
    port_number: int
    ip_address: IPvAnyAddress
    ip_address_unique_id: Optional[
        UUID
    ] = None  # for private IPs, uniquely identify the IP to avoid collisions

    unique_id: Optional[UUID] = None

    def get_identifier(self) -> str:
        return f"{self.ip_address}:{self.port_number}"

    @validator("unique_id", always=True)
    def generate_socket_uuid(cls, v: Optional[UUID], values: Dict[str, Any]) -> UUID:

        if v is None:

            key_values = [
                values["protocol"],
                values["port_number"],
                values["ip_address"],
                values["ip_address_unique_id"],
            ]

            v = generate_deterministic_uuid(key_values)

        return v


#
# OUTGOING RELATIONSHIPS
#


class ListeningSocketUsesPort(OntolocyRelationship):
    source: ListeningSocket
    target: Port

    __relationshiptype__: ClassVar[str] = "LISTENING_SOCKET_USES_PORT"


class OpenPortPresentsBanner(OntolocyRelationship):
    source: ListeningSocket
    target: Banner

    __relationshiptype__: ClassVar[str] = "OPEN_PORT_PRESENTS_BANNER"


class OpenPortPresentsX509Certificate(OntolocyRelationship):
    source: ListeningSocket
    target: X509Certificate

    __relationshiptype__: ClassVar[str] = "OPEN_PORT_PRESENTS_X509_CERTIFICATE"


class OpenPortHasJarmHash(OntolocyRelationship):
    source: ListeningSocket
    target: JarmHash

    __relationshiptype__: ClassVar[str] = "OPEN_PORT_HAS_JARM_HASH"


class ServiceHostsURL(OntolocyRelationship):
    source: ListeningSocket
    target: URLNode
    status_code: Optional[int]

    __relationshiptype__: ClassVar[str] = "SERVICE_HOSTS_URL"


class ServiceIdentifiedAsPlatform(OntolocyRelationship):
    source: ListeningSocket
    target: CPE
    status_code: Optional[int]

    __relationshiptype__: ClassVar[str] = "SERVICE_IDENTIFIED_AS_PLATFORM"



from typing import Any, ClassVar, Dict, Optional
from uuid import UUID

from neontology import BaseNode, BaseRelationship
from pydantic import validator

from .utils import generate_deterministic_uuid


class DataOrigin(BaseNode):

    __primaryproperty__: ClassVar[str] = "unique_id"
    __primarylabel__: ClassVar[Optional[str]] = "DataOrigin"

    name: str
    reference: Optional[str] = None
    license: Optional[str] = None
    sharing: Optional[str] = None

    unique_id: Optional[UUID] = None

    @validator("unique_id", always=True)
    def generate_unique_id(cls, v: Optional[UUID], values: Dict[str, Any]) -> UUID:

        if v is None:

            key_values = [
                values["name"],
                values["reference"],
                values["license"],
                values["sharing"],
            ]

            v = generate_deterministic_uuid(key_values)

        return v


class OriginGenerated(BaseRelationship):

    __relationshiptype__: ClassVar[Optional[str]] = "ORIGIN_GENERATED"

    source: DataOrigin
    target: BaseNode
